"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var message_bus_service_1 = require("../services/message-bus.service");
var alert_service_1 = require("../services/alert.service");
var logging_service_1 = require("../services/logging.service");
var modal_service_1 = require("../services/modal.service");
var angular_service_1 = require("../services/angular.service");
var angular_base_controller_1 = require("./angular-base.controller");
var ControllerBase = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerBase, _super);
    function ControllerBase(scope, injector) {
        var _this = _super.call(this, scope) || this;
        _this.injector = injector;
        _this.initializeServices();
        return _this;
    }
    ControllerBase.prototype.initializeServices = function () {
        this.logger = this.getService(logging_service_1.LoggingService);
        this.alertService = this.getService(alert_service_1.AlertService);
        this.stateService = this.getService(angular_service_1.AngularServices.state);
        this.messageBus = this.getService(message_bus_service_1.MessageBus);
        this.translator = this.getService(angular_service_1.AngularServices.translate);
        this.messageBusTokens = new paradigm_ui_web_shared_1.ArrayList();
    };
    ControllerBase.prototype.onDestroy = function () {
        _super.prototype.onDestroy.call(this);
        this.unregisterAllMessages();
    };
    ControllerBase.prototype.getService = function (service) {
        return this.injector.get(service);
    };
    ControllerBase.prototype.open = function (controller, parameters, staticDialog, keyboard) {
        return this.getService(modal_service_1.ModalService).open(controller, parameters, staticDialog, keyboard);
    };
    ControllerBase.prototype.translate = function (key) {
        return this.translator.instant(key);
    };
    ControllerBase.prototype.call = function (call, success, loading, fail) {
        var _this = this;
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(loading))
            loading(true);
        call()
            .then(function (result) {
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(loading))
                loading(false);
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(success))
                success(result);
        })
            .catch(function (error) {
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(loading))
                loading(false);
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(fail))
                fail(error);
            _this.handleException(error);
        });
    };
    ControllerBase.prototype.showErrors = function (messages) {
        if (messages.length === 0)
            return;
        for (var i = 0; i < messages.length; i++) {
            this.showError(messages[i]);
        }
    };
    ControllerBase.prototype.showWarnings = function (messages) {
        if (messages.length === 0)
            return;
        for (var i = 0; i < messages.length; i++) {
            this.showWarning(messages[i]);
        }
    };
    ControllerBase.prototype.showError = function (message) {
        this.alertService.addError(message);
    };
    ControllerBase.prototype.showWarning = function (message) {
        this.alertService.addWarning(message);
    };
    ControllerBase.prototype.showMessage = function (message) {
        this.alertService.addMessage(message);
    };
    ControllerBase.prototype.handleException = function (ex) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(ex))
            return;
        // if ex.data is not null, it's probably a
        // an http promise exception.
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(ex.data))
            ex = ex.data;
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(ex.message)) {
            this.showError(ex.message);
        }
        else if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(ex.Message)) {
            this.showError(ex.Message);
        }
        else if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(ex.ExceptionMessage)) {
            this.showError(ex.ExceptionMessage);
        }
        else if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(ex.error) && !paradigm_ui_web_shared_1.ObjectExtensions.isNull(ex.error.message)) {
            this.showError(ex.error.message);
        }
    };
    ControllerBase.prototype.changeState = function (state, params, reload) {
        if (reload === void 0) { reload = false; }
        return this.stateService.go(state, params, { reload: reload });
    };
    ControllerBase.prototype.registerMessage = function (messageType, handler) {
        var token = this.messageBus.register(messageType, handler);
        this.messageBusTokens.add(token);
        return token;
    };
    ControllerBase.prototype.sendMessage = function (message, token) {
        return this.messageBus.send(message);
    };
    ControllerBase.prototype.unregisterMessage = function (token) {
        if (!this.messageBusTokens.any(function (x) { return x.guid === token.guid; }))
            throw new Error("Can not unregister a token registered outside this controller.");
        this.messageBus.unregister(token);
        this.messageBusTokens.removeAll(function (x) { return x.guid === token.guid; });
    };
    ControllerBase.prototype.unregisterAllMessages = function () {
        var _this = this;
        this.messageBusTokens.forEach(function (x) { return _this.messageBus.unregister(x); });
    };
    return ControllerBase;
}(angular_base_controller_1.AngularControllerBase));
exports.ControllerBase = ControllerBase;
