"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var registrable_1 = require("./registrable");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
function addBinding(binding, target, propertyKey) {
    if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(target) ||
        paradigm_ui_web_shared_1.ObjectExtensions.isNull(target.constructor) ||
        paradigm_ui_web_shared_1.ObjectExtensions.isNull(propertyKey))
        throw new Error("Missing target or propery.");
    var constructor = target.constructor;
    var parameters = constructor[registrable_1.registrableMetadataKey];
    if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters))
        parameters = constructor[registrable_1.registrableMetadataKey] = {};
    if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters.bindings))
        parameters.bindings = {};
    parameters.bindings[propertyKey] = binding;
}
function Component(parameters) {
    if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters))
        throw new Error("Can not register a component without an instance of IComponent.");
    return function (constructor) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(constructor))
            throw new Error("Missing component constructor.");
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(constructor[registrable_1.registrableMetadataKey])) {
            var params = constructor[registrable_1.registrableMetadataKey];
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(params.bindings))
                parameters.bindings = params.bindings;
            if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(params.require))
                parameters.require = params.require;
        }
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters.name))
            parameters.name = paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(constructor);
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters.controllerAs))
            parameters.controllerAs = "controller";
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters.controller))
            parameters.controller = constructor;
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters.controller))
            throw new Error("Missing component constructor.");
        constructor[registrable_1.registrableMetadataKey] = parameters;
        if (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters.module)) {
            parameters.module.registerComponent(constructor);
        }
    };
}
exports.Component = Component;
function InputBinding(target, propertyKey) {
    addBinding("<", target, propertyKey);
}
exports.InputBinding = InputBinding;
function OutputBinding(target, propertyKey) {
    addBinding("&", target, propertyKey);
}
exports.OutputBinding = OutputBinding;
function TextBinding(target, propertyKey) {
    addBinding("@", target, propertyKey);
}
exports.TextBinding = TextBinding;
function TwoWayBinding(target, propertyKey) {
    addBinding("=", target, propertyKey);
}
exports.TwoWayBinding = TwoWayBinding;
function Require(controller, searchOnParents) {
    if (searchOnParents === void 0) { searchOnParents = false; }
    return function (target, propertyKey) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(target) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(target.constructor) ||
            paradigm_ui_web_shared_1.ObjectExtensions.isNull(propertyKey))
            throw new Error("Missing target or propery.");
        var constructor = target.constructor;
        var parameters = constructor[registrable_1.registrableMetadataKey];
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters))
            parameters = constructor[registrable_1.registrableMetadataKey] = {};
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(parameters.require))
            parameters.require = {};
        var controllerName = (paradigm_ui_web_shared_1.ObjectExtensions.getTypeName(controller) === "Function")
            ? paradigm_ui_web_shared_1.FunctionExtensions.getFunctionName(controller)
            : controller;
        if (searchOnParents)
            controllerName = "^" + controllerName;
        parameters.require[propertyKey] = controllerName;
    };
}
exports.Require = Require;
