"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var service_1 = require("../decorators/service");
var base_service_1 = require("./base.service");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var message_1 = require("../decorators/message");
var MessageBus = /** @class */ (function (_super) {
    tslib_1.__extends(MessageBus, _super);
    function MessageBus() {
        var _this = _super.call(this) || this;
        _this.handlers = new paradigm_ui_web_shared_1.Dictionary();
        return _this;
    }
    MessageBus_1 = MessageBus;
    MessageBus.prototype.count = function () {
        return this.handlers.count();
    };
    MessageBus.prototype.isRegistered = function (messageType) {
        var type = messageType[message_1.messageTypeKey];
        return this.handlers.containsKey(type);
    };
    MessageBus.prototype.handlerCount = function (messageType) {
        var type = messageType[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            return 0;
        return this.handlers.get(type).count();
    };
    MessageBus.prototype.register = function (messageType, handler) {
        var type = messageType[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            this.handlers.add(type, new paradigm_ui_web_shared_1.ArrayList());
        var messageBusHandler = new MessageBusHandler(messageType, handler);
        this.handlers.get(type).add(messageBusHandler);
        return messageBusHandler.token;
    };
    MessageBus.prototype.unregister = function (token) {
        var type = token.type[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            return;
        var handler = this.handlers.get(type);
        handler.removeAll(function (x) { return x.token.guid === token.guid; });
        if (!handler.any())
            this.handlers.remove(type);
    };
    MessageBus.prototype.send = function (message, token) {
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(message))
            throw new Error("Can not send a null message.");
        if (paradigm_ui_web_shared_1.ObjectExtensions.isNull(message.constructor))
            throw new Error("Message does not have a constructor, and the system can not infer the type.");
        var type = message.constructor[message_1.messageTypeKey];
        if (!this.handlers.containsKey(type))
            return false;
        var handlers = this.handlers.get(type)
            .where(function (x) { return token == null || token.guid === x.token.guid; })
            .select(function (x) { return x.handler; });
        var accepted = false;
        for (var i = (handlers.count() - 1); i >= 0; i--) {
            var handler = handlers.get(i);
            if (handler == null)
                continue;
            accepted = true;
            handler(message);
        }
        return accepted;
    };
    MessageBus.factory = function () {
        return new MessageBus_1();
    };
    MessageBus = MessageBus_1 = tslib_1.__decorate([
        service_1.Service({ name: "$pd-messageBus" })
    ], MessageBus);
    return MessageBus;
    var MessageBus_1;
}(base_service_1.ServiceBase));
exports.MessageBus = MessageBus;
var MessageBusHandler = /** @class */ (function () {
    function MessageBusHandler(type, handler) {
        this.innerHandler = handler;
        this.innerToken = new RegistrationToken(type);
    }
    Object.defineProperty(MessageBusHandler.prototype, "token", {
        get: function () { return this.innerToken; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MessageBusHandler.prototype, "handler", {
        get: function () { return this.innerHandler; },
        enumerable: true,
        configurable: true
    });
    return MessageBusHandler;
}());
exports.MessageBusHandler = MessageBusHandler;
var RegistrationToken = /** @class */ (function () {
    function RegistrationToken(type) {
        this.innerType = type;
        this.innerGuid = paradigm_ui_web_shared_1.Guid.new();
    }
    Object.defineProperty(RegistrationToken.prototype, "type", {
        get: function () {
            return this.innerType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RegistrationToken.prototype, "guid", {
        get: function () {
            return this.innerGuid;
        },
        enumerable: true,
        configurable: true
    });
    return RegistrationToken;
}());
exports.RegistrationToken = RegistrationToken;
